package com.streambase.contrib.stats;

import java.util.List;

import com.streambase.sb.CompleteDataType;
import com.streambase.sb.DataType;
import com.streambase.sb.client.CustomFunctionResolver;

public class RandomVariables {

	@CustomFunctionResolver("GenerateCustomFunctionResolver0")
	public static String GenerateRandom(String Distribution, int Number, List<Double> Parameters) {
		String d = Distribution.toLowerCase();
		int size = Parameters.size();
		if (Number < 1) {return null;}
		
		// Beta Distribution: Parameters
		// (Number of observations, Shape parameter 1, Shape parameter 2, Non-centrality parameter)
		if (d.equals("beta")){
			if (size == 2){
				return String.format("rbeta(%d,%f,%f)",Number,Math.abs(Parameters.get(0)),Math.abs(Parameters.get(1)));
			} else if(size == 3){
				return String.format("rbeta(%d,%f,%f,%f)",Number,Math.abs(Parameters.get(0)),Math.abs(Parameters.get(1)),Math.abs(Parameters.get(2)));
			}

		// Binomial Distribution:
		// (Number of observations, Number of trials (zero or more), Probability of a success on each trial)			
		} else if (d.equals("binomial")){
			if(size == 3){
				return String.format("rbinom(%d,%f,%f)",Number,Parameters.get(0),Math.abs(Parameters.get(1)));
			}

		// Cauchy Distribution
		// (Number of observations, Location Parameter, Scale parameter
		} else if (d.equals("cauchy")){
			if(size == 0){
				return String.format("rcauchy(%d)",Number);
			} else if(size == 1){
				return String.format("rcauchy(%d,%f)",Number,Math.abs(Parameters.get(0)));
			} else if(size == 2){
				return String.format("rcauchy(%d,%f,%f)",Number,Math.abs(Parameters.get(0)),Math.abs(Parameters.get(1)));
			}
			 
		// Chi-Squared Distribution
		// (Number of observations, Degrees of freedom, Non-centrality parameter)
		} else if (d.equals("chisquared")){
			if(size == 1){
				return String.format("rchisq(%d,%f)",Number,Math.abs(Parameters.get(0)));
			} else if(size == 2){
				return String.format("rchisq(%d,%f,%f)",Number,Math.abs(Parameters.get(0)),Math.abs(Parameters.get(1)));
			}			
			
		// Exponential Distribution	
		// (Number of observations, Rate parameter)
		} else if (d.equals("exponential")){
			if(size == 0){
				return String.format("rexp(%d)",Number);
			} else if(size == 1){
				return String.format("rexp(%d,%f)",Number,Math.abs(Parameters.get(0)));
			} 
		
		// F-Distribution
		// (Number of observations, Degrees of freedom 1, Degrees of freedom 2, Non-centrality parameter)
		} else if (d.equals("f")){
			if(size == 2){
				return String.format("rf(%d,%f,%f)",Number,Math.abs(Parameters.get(0)),Math.abs(Parameters.get(1)));
			} else if(size == 3){
				return String.format("rf(%d,%f,%f,%f)",Number,Math.abs(Parameters.get(0)),Math.abs(Parameters.get(1)),Math.abs(Parameters.get(2)));
			}	

		// Gamma Distribution	
		// (Number of observations, Rate parameter, shape parameter, scale parameter)
		} else if (d.equals("gamma")){
			if(size == 1){
				return String.format("rgamma(%d,%f,)",Number,Math.abs(Parameters.get(0)));
			} else if(size == 2){
				return String.format("rgamma(%d,%f,%f)",Number,Math.abs(Parameters.get(0)),Math.abs(Parameters.get(1)));	
			} else if(size == 3){
				return String.format("rgamma(%d,%f,%f,%f)",Number,Math.abs(Parameters.get(0)),Math.abs(Parameters.get(1)),Math.abs(Parameters.get(2)));
			}	
		
		// Geometric Distribution
		// (Number of observations, probability of a success in each trial)
		} else if (d.equals("geometric")){
			if(size == 0){
				return String.format("rgeom(%d)",Number);
			} else if(size == 1){
				return String.format("rgeom(%d,%f)",Number,Math.abs(Parameters.get(0)));
			} 

		// Lognormal Distribution
		// (Number of observations, mean on the log scale, standard deviation on the log scale)		
		} else if (d.equals("lognormal")){
			if(size == 0){
				return String.format("rlnorm(%d)",Number);
			} else if(size == 1){
				return String.format("rlnorm(%d,%f)",Number,Math.abs(Parameters.get(0)));
			} else if(size == 2){
				return String.format("rlnorm(%d,%f,%f)",Number,Math.abs(Parameters.get(0)),Math.abs(Parameters.get(1)));
			}
			
		// Negative Binomial Distribution
		// (Number of observations, target for number of successful trials, probability of a success in each trial)				
		} else if (d.equals("negativebinomial")){
			if(size == 2){
				return String.format("rnbinom(%d,%f,%f)",Number,Math.abs(Parameters.get(0)),Math.abs(Parameters.get(1)));
			}
			
		// Normal Distribution
		// (Number of observations, mean, standard deviation)	
		} else if (d.equals("normal")){			
			if(size == 0){				
				return String.format("rnorm(%d)",Number);
			} else if(size == 1){
				return String.format("rnorm(%d,%f)",Number,Math.abs(Parameters.get(0)));
			} else if(size == 2){
				return String.format("rnorm(%d,%f,%f)",Number,Math.abs(Parameters.get(0)),Math.abs(Parameters.get(1)));
			}
			
		// Poisson Distribution
		// (Number of observations, rate parameter)
		} else if (d.equals("poisson")){
			if(size == 1){
				return String.format("rpois(%d,%f)",Number,Math.abs(Parameters.get(0)));
			}
			
		// Student T distribution
		// (Number of observations, Degrees of freedom, Non-centrality parameter)			
		} else if (d.equals("t")){		
			if(size == 1){
				return String.format("rt(%d,%f)",Number,Math.abs(Parameters.get(0)));
			} else if(size == 2){
				return String.format("rt(%d,%f,%f)",Number,Math.abs(Parameters.get(0)),Math.abs(Parameters.get(1)));
			}		
		
		// Uniform Distribution
		// (Number of observations, Lower limit of interval, Upper limit of interval)
		} else if (d.equals("uniform")){
			if(size == 0){
				return String.format("runif(%d)",Number);
			} else if(size == 1){
				return String.format("runif(%d,%f)",Number,Math.abs(Parameters.get(0)));
			} else if(size == 2){
				return String.format("runif(%d,%f,%f)",Number,Math.abs(Parameters.get(0)),Math.abs(Parameters.get(1)));
			}
		
		// Weibull Distribution
		// (Number of observations, Shape parameter, Scale parameter)
		} else if (d.equals("weibull")){
			if(size == 1){
				return String.format("rweibull(%d,%f)",Number,Math.abs(Parameters.get(0)));
			} else if(size == 2){
				return String.format("rweibull(%d,%f,%f)",Number,Math.abs(Parameters.get(0)),Math.abs(Parameters.get(1)));
			}
		
		// Wilcox Distribution
		// (Number of observations, number of observations in the first sample, number of observations in the second sample)
		} else if (d.equals("wilcox")){
			if(size == 2){
				return String.format("rwilcox(%d,%f,%f)",Number,Math.abs(Parameters.get(0)),Math.abs(Parameters.get(1)));
			}
			
		// Logistic Distribution
		// (Number of observations, Location parameter, Scale parameter)			
		} else if (d.equals("logistic")){
			if(size == 0){
				return String.format("rlogis(%d)",Number);
			} else if(size == 1){
				return String.format("rlogis(%d,%f)",Number,Math.abs(Parameters.get(0)));
			} else if(size == 2){
				return String.format("rlogis(%d,%f,%f)",Number,Math.abs(Parameters.get(0)),Math.abs(Parameters.get(1)));
			}
		}
		return null;
	}

	public static CompleteDataType GenerateCustomFunctionResolver0(
			CompleteDataType Distribution, CompleteDataType Number, CompleteDataType Parameters) {
		CompleteDataType p = Parameters.getElementType();
		if (Parameters == null){return null;}
		if (Distribution.getDataType() != DataType.STRING || Number.getDataType() != DataType.INT) {return null;}
		if (p.getDataType() != DataType.DOUBLE) {return null;}
		return CompleteDataType.forString();
	}

}
